///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#ifndef READOUT_H
#define READOUT_H

#include "constant.h"

class Cluster;
class Field;
class Inparam;
class Point;

//
//  class readout will handle avalanche, pad/shaper response 
//  and digitization
//
class Readout
{
    public:
              Readout(const Inparam& inp);
              Readout(const Inparam& inp,
                      const int nplane, const fNumber pitch, 
                      const fNumber length, const fNumber prf, 
                      const fNumber shaper, const fNumber slice);
              ~Readout();
         void Avalanche(const Cluster *cl, const Field *field);      
                                // electron multiplification
         void PadResponse(const Cluster *cl);    
                                // width including pad response in [um]
         void ShaperResponse(const Cluster *cl, const Field *field); 
                                // width including shaper response in [nsec]
         void Digitize(const Cluster *cl, const Chamber *ftpc);       
                                // add noise and fill the pixel array
                                //     ADC[pad,time]
         void BackProject(const Field* field, const int i, 
                          Point* point);
                                // project cluster back to hit point since 
                                // we don't have a cluster-finder yet !!
         void OutputADC(const char* file_adc) const ;
         void Print() const ;   // print readout parameters

         void SetRandomNumberGen(const int rng) {ran_num_gen = rng;}

         void SetNumberPlane(const int nplane) {number_plane = nplane;}
         void SetNPad(const Inparam& inp);
         void SetPadPitch(const fNumber pitch) {pad_pitch = pitch;}
         void SetPadLength(const fNumber length) {pad_length = length;}
         void SetPadResponse(const fNumber prf)  {sigma_prf = prf;}
         void SetShaperTime(const fNumber time)  {shaper_time = time;}
         void SetTimeSlice(const fNumber tslice) {slice = tslice;}

          int GetRandomNumberGen() const  {return ran_num_gen; }

          int GetNumberPlane() const      {return number_plane; }
          int GetNPad() const             {return NPAD;}
      fNumber GetPadPitch()  const        {return pad_pitch;}
      fNumber GetPadLength() const        {return pad_length; }
      fNumber GetPadResponse() const      {return sigma_prf; }   
      fNumber GetShaperTime() const       {return shaper_time; }
      fNumber GetShaperResponse() const   {return shaper_time/2.35482; }
      fNumber GetTimeSlice() const        {return slice;}

      fNumber GetDeltaPhi(const int deg_or_rad, const Chamber *ftpc);
                                           // delta phi for one pad
          int WhichSlice(const fNumber time);    // which time slice
          int WhichPad(const fNumber phi, const Chamber *ftpc);       
                                           // which pad is phi
      fNumber TimeOfSlice(const int slice);// time for slice in nsec
      fNumber PhiOfPad(const int pad, const int deg_or_rad, 
                       const Chamber *ftpc);
                                           // phi for pad
                                           // deg_or_rad = 1: in deg
                                           //              0: in rad
      fNumber GetAElectron() const        {return electrons;}
      fNumber GetSigmaPad() const         {return sigma_pad;}
      fNumber GetSigmaTim() const         {return sigma_tim;}

    private:
          int ran_num_gen;      // which random number generator
          int NPAD;             // number of pads
        //int NTIM;             // number of time slice given in constant.h
     fNumber* ADC;              // pixel array (fNumber)
                                // to get integer ADC, one simply
                                // scale it by a constant. we worry it later.
                                // NPAD,NTIM should be changed too.

      fNumber electrons;        // number of electrons after avalanche
      fNumber sigma_pad;        // azimuthal width after pad response in um
      fNumber sigma_tim;        // radial  width after shaper response  in nsec

          int number_plane;     // number of readout planes
      fNumber pad_pitch;        // pad pitch in cm
      fNumber pad_length;       // pad length in cm 
      fNumber sigma_prf;        // pad-response-function sigma in um 
      fNumber shaper_time;      // FWHM of shaper time in nsec
      fNumber slice;            // SCA time slice in nsec

     fNumber* pcum;             // cumulative Polya dist
          int gnch;             // No. of bins
      fNumber glow;             // lower bound
      fNumber ghigh;            // upper bound
      fNumber gdelta;           // bin size
         void polya(const int nch, const fNumber low, 
                    const fNumber high, const fNumber delta);
                                // get pcum[]
          int sample_polya(const fNumber gain);
                                // sample gas gain for the averaged gain

};

#endif

