
///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#include "constant.h"
#include "inparam.h"
#include "chamber.h"

Chamber::Chamber(const Inparam& inp) 
{
    SetRMin( inp.GetInParam(Chamber_r_min) );
    SetRMax( inp.GetInParam(Chamber_r_max) );
    SetPhiMin( inp.GetInParam(Chamber_phi_min), 1 );
    SetPhiMax( inp.GetInParam(Chamber_phi_max), 1 );
    SetZMin( inp.GetInParam(Chamber_z_min) );
    SetChamberLength( inp.GetInParam(Chamber_cham_length) );
    SetCathodeVoltage( inp.GetInParam(Chamber_cath_voltage) );
}

Chamber::Chamber(const fNumber rmin, const fNumber rmax, 
                 const fNumber phimin, const fNumber phimax,
                 const fNumber zmin, const fNumber length, 
                 const fNumber voltage)
{
    SetRMin(rmin);
    SetRMax(rmax);
    SetPhiMin(phimin,1);
    SetPhiMax(phimax,1);
    SetZMin(zmin);
    SetChamberLength(length);
    SetCathodeVoltage(voltage);
}
 
Chamber::~Chamber() {}

void Chamber::SetPhiMin(const fNumber phi, const int deg_or_rad=1)
{
    phi_min = deg_or_rad ?  phi* PID       // in deg
                         :  phi;           // in rad
}

void Chamber::SetPhiMax(const fNumber phi, const int deg_or_rad=1)
{
    phi_max = deg_or_rad ?  phi* PID       // in deg
                         :  phi;           // in rad
}

void Chamber::SetZMax(const fNumber z)
{
    if( defined != 1) {
         z_max = z;
         cham_length = z_max - z_min; 
         defined = 1; 
    }
}

void Chamber::SetCathodeVoltage(const fNumber voltage)
{
    cath_voltage = voltage;
  
    if (voltage > 0) 
        drift_direction = -1;
    else if (voltage < 0) 
        drift_direction = +1;
    else
        drift_direction = 0;

}

void Chamber::SetChamberLength(const fNumber length)
{
    if ( defined != 1) {
         cham_length = length;
         z_max = z_min + cham_length;
         defined = 1;
    }
}

fNumber Chamber::GetPhiMin(const int deg_or_rad=1) const 
{
    return deg_or_rad ? phi_min*180/M_PI : phi_min ; 
}

fNumber Chamber::GetPhiMax( const int deg_or_rad=1) const 
{
    return deg_or_rad ? phi_max*180/M_PI : phi_max ; 
}

fNumber Chamber::GetRAtReadout() const 
{
    if ( drift_direction == -1 ) 
        return r_min;
    else 
        return r_max;
}

void Chamber::Print() const 
{
    cout << "The chamber parameters are defined as follows:" << endl;
    cout << "=======================================" << endl;
    cout << " Radius = " << GetRMin() << " (inner) "
         << GetRMax() << " (outer) [cm] " << endl;
    cout << " Angle  = " << GetPhiMin(1) << " (min) "
         << GetPhiMax(1) << " (max) [deg] " << endl;
    cout << " Z      = " << GetZMin() << " (entrance)"
         << GetZMax() << " (exit) [cm] " << endl;
    cout << " Chamber length  = " << GetChamberLength()
         << " [cm]" << endl;
    cout << " Cathode voltage = " << GetCathodeVoltage()
         << " [V] " << endl;
}


void Chamber::Initialize()
{
  // to begin with, we set chamber parameters by hand.
  // later on, we will parse the GStar geometry definition file
  // to automate the modification and reflect the current definition
  //
    SetRMin(8.0);
    SetRMax(32.0);
    SetPhiMin(0.);
    SetPhiMax(40.);
    SetZMin(150.);
    SetChamberLength(120.);
    SetCathodeVoltage(10000.);
}

