///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#include "constant.h"
#include "cluster.h"
#include "field.h"

Cluster::Cluster(const fNumber el, const fNumber sig_rad, 
                 const fNumber sig_phi, const fNumber r, 
                 const fNumber ph, const fNumber time)
{
    electron     = el;
    sigma_rad    = sig_rad;
    sigma_phi    = sig_phi;
    rad          = r;
    phi          = ph;
    drift_time   = time;
}

Cluster::~Cluster() {}

void Cluster::DriftDiffuse(const Field *field)
{      
    fNumber del_r = field->GetDeltaR();
    int     index = field->GetGridIndex(rad);

    // deflection angle
    phi          += fabs(del_r) / rad * field->GetLorentzAngle(index);

    // accumulative drift time
    drift_time   += fabs(del_r) / field->GetVelocityZ(index);

    // longitudinal diffusion + velocity gradience
    fNumber dz     = field->GetDiffusionZ(index);
    fNumber dlnvdr = field->GetDlnvDr(index);
    sigma_rad      = sqrt( dz*dz*fabs(del_r) + 
             sigma_rad*sigma_rad*(1.+2.*dlnvdr*del_r) );
     
    // transverse  diffusion + radius divergence
    fNumber dx     = field->GetDiffusionX(index);
    sigma_phi      = sqrt(dx*dx*fabs(del_r) +
             sigma_phi*sigma_phi*(1.+del_r/rad)*(1.+del_r/rad) );

    // attenuation
    electron    -= fabs(del_r) * field->GetAttenuation();

    // next step
    rad         += del_r; 

}

int Cluster::IsDrifting(const Field* field)
{
    int flag = 1;      // indicate el drifting
    if ( (field->GetDriftDirection() == +1) &&
          GetRad() >= field->GetRMax() ) flag = 0;
    if ( (field->GetDriftDirection() == -1) &&
          GetRad() <= field->GetRMin() ) flag = 0;
    return flag;
}

void Cluster::Print() const
{
    cout << " Cluster parameter : " << endl;
    cout << "number of electrons = " << electron << endl;
    cout << "sigma_rad [um]      = " << sigma_rad << endl;
    cout << "sigma_phi [um]      = " << sigma_phi << endl;
    cout << "phi [deg]   = " << GetPhi(1) << endl;
    cout << "rad [cm]    = " << rad << endl;
    cout << "Td [usec]   = " << drift_time << endl;
}
