///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#include "constant.h"
#include "inparam.h"
#include "gas.h"
#include "chamber.h"
#include "field.h"

Field::Field(Inparam& inp)
      :Chamber(inp),
       Gas(inp)
{
    DefineField();
}

Field::Field(const fNumber rmin, const fNumber rmax, 
             const fNumber phimin, const fNumber phimax,
             const fNumber zmin, const fNumber length, 
             const fNumber voltage, const fNumber attenu, 
             const fNumber gain, const fNumber aip, 
             const char* file_gas) 
      :Chamber(rmin, rmax, phimin, phimax, zmin, length, voltage),
       Gas(attenu, gain, aip, file_gas) 
{
    DefineField();
}

Field::~Field() {}

int Field::GetGridIndex(const fNumber r) const
{
    return ((int) ((r - GetRMin())/del_r) ) ;
}

fNumber Field::GetVelAtReadout() const
{
    // get the drift velocity at Readout radius in [um/nsec]
    fNumber vout;
    int direction = GetDriftDirection();

    if (direction == 1 ) 
        vout = grid_point[NGRID-1].vel_z*10.;
    else
        vout = grid_point[0].vel_z*10.;

    return  vout;
}

fNumber Field::GetDeltaR() const
{
    return ( GetDriftDirection() * del_r );
}

fNumber Field::RField(const fNumber r)
{
    return  fabs(GetCathodeVoltage()) / 
            log(GetRMax()/GetRMin())/r  ;
}

void Field::DefineField()
{
    int i;
    // grid size
    del_r = (GetRMax() - GetRMin())/ (fNumber) (NGRID-1);

    int num_data = GetNGas();    // number of data point
    fNumber* in_efield      = new fNumber[num_data];
    fNumber* in_velocity_z  = new fNumber[num_data];
    fNumber* in_diffusion_z = new fNumber[num_data];
    fNumber* in_diffusion_x = new fNumber[num_data];
    fNumber* in_diffusion_y = new fNumber[num_data];
    fNumber* in_angle_lorentz = new fNumber[num_data];
    for ( i=0; i<num_data; i++) {
        in_efield[i]        = GetInEfield(i);
        in_velocity_z[i]    = GetInVelocityZ(i);
        in_diffusion_z[i]   = GetInDiffusionZ(i);
        in_diffusion_x[i]   = GetInDiffusionX(i);
        in_diffusion_y[i]   = GetInDiffusionY(i);
        in_angle_lorentz[i] = GetInAngleLorentz(i);
    }

    fNumber eff;
    // big loop to define the field
    for ( i=0; i<NGRID; i++) {

        grid_point[i].rhit = GetRMin() + i * del_r;

        eff = RField(grid_point[i].rhit);
        grid_point[i].ef   = eff;

        int index = Locate(num_data, in_efield, eff);

        grid_point[i].vel_z   
                  = Interpolate(num_data,in_efield,in_velocity_z,index,eff);
        grid_point[i].diff_z  
                  = Interpolate(num_data,in_efield,in_diffusion_z,index,eff);
        grid_point[i].diff_x  
                  = Interpolate(num_data,in_efield,in_diffusion_x,index,eff);
        grid_point[i].diff_y  
                  = Interpolate(num_data,in_efield,in_diffusion_y,index,eff);
        grid_point[i].lorentz 
                  = Interpolate(num_data,in_efield,in_angle_lorentz,index,eff);
    }

    // calculate drift velocity gradients

    for (i=NGRID-1; i>0; i--) {
        grid_point[i].dlnv_dr 
                  = ( 1. - grid_point[i-1].vel_z/grid_point[i].vel_z)/del_r;
    }
    grid_point[0].dlnv_dr = grid_point[1].dlnv_dr; 

}

void Field::Output() const
{
    ofstream fout("field.dat");
    char space = ' ';
    // print out the field value to check
    for (int  i=0; i<NGRID; i++) {
       fout << grid_point[i].rhit << space 
            << grid_point[i].ef << space 
            << grid_point[i].vel_z << space
            << grid_point[i].diff_z<< space 
            << grid_point[i].diff_x<< space 
            << grid_point[i].lorentz*180/M_PI << space 
            << grid_point[i].dlnv_dr 
            << endl;
    }
}
