///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////
//
// This is the main routine  for the FTPC Simulator
//
#include "constant.h"
#include "inparam.h"
#include "point.h"
#include "chamber.h"
#include "gas.h"
#include "field.h"
#include "cluster.h"
#include "readout.h"

static char file_config[MAXCH];
static char file_chamber[MAXCH];
static char file_cluster[MAXCH];
static char file_adc[MAXCH];

void Help()
{
    cout << "Usuage for FTPC slow simulator:: " << endl;
    cout << " ftpcsim -h[elp] -i[nput] <config-file> -d[ebug] " << endl;
    cout << " ------------------------------------------------" << endl;
    cout << "   -h[elp]   ---  print this message " << endl;
    cout << "   -d[ebug]  ---  print out some variables " << endl;
    cout << "   -i[nput]  ---  give the config. file to setup input" << endl;
    cout << "             ---  parameters for chamber, gas, etc " << endl;
    exit(1);
}


int main(int argc, char **argv)
{
    int CONFIG  = 0;
    int DEBUG   = 0;
    int CLUSTER = 0;
    int PIXEL   = 0;

    int i;

    // parsing command line inputs
    if ( argc < 3 ) {
        Help();
    }
    else {
        char option;
        i = 1;
        while (i<argc) {
            option = argv[i][0];
            if ( option != '-' ) Help();
            else {
                option = argv[i][1];
                switch (option) {
                    case 'd': 
                        //cout << " case_"<<option << endl;
                        DEBUG = 1;
                        ++i;
                        break;
                    case 'i':
                        //cout << " case_"<<option << endl;
                        CONFIG = 1;
                        cout << " argv= "<<argv[i+1] << endl;
                        strcpy(file_config, argv[i+1]);
                        //cout << " file_config= "<<file_config << endl;
                        i += 2;
                        break;
                    default:
                        //cout << " case_"<<option << endl;
                        Help();
                        ++i;
                }  // end switch option
            }  // end if not '-'
        }  // end while
    }  // end if checking argc

    if(!CONFIG) Help();

    cout << " begin the program " << endl;

    // read config file to fetch input parameters
    Inparam inp(file_config, DEBUG);

    strcpy(file_adc,     inp.GetFileName(Readout_file_adc));
    if(strcmp(file_adc, "no_output")) PIXEL = 1;

    strcpy(file_cluster, inp.GetFileName(Readout_file_cluster));
    if(strcmp(file_cluster, "no_output")) CLUSTER = 1;

    cout << "Begin Initialization..." << endl;

    // setup chamber 
    Chamber *ftpc = new Chamber(inp);
    if( !(ftpc->GetDriftDirection()) ) {
        cerr << "drift_direction = 0, cannot drift !" << endl;
        exit(2);
    }
    if(DEBUG) ftpc->Print();

    // setup fields
    Field *field = new Field(inp);              // define the field
    if(DEBUG) field->Output();                  // write out field 

    // setup readout
    Readout *rdout = new Readout(inp);          // create readout
    if(DEBUG) rdout->Print();

    /////////////////////////////////////
    // big loop over all the hit points
    /////////////////////////////////////
    cout << "Looping over points..." << endl;

    // setup hit points
    Point *point = new Point(inp, DEBUG);

    // tmp variables
    fNumber electron;      
    fNumber sig_rad;
    fNumber sig_phi;
    fNumber rad;
    fNumber phi;
    fNumber drift_time;

    fNumber sig0_pad = rdout->GetPadLength()/sqrt(12.)*10000; // in um
    fNumber aip = field ->GetAvgIonPot()*1.0e-9;              // in GeV

    for( i=0; i < point->GetNumberPoint(); ++i ) {
         
        // define cluster for each point
        drift_time  = 0.;                          
        electron    = point->GetDe(i) / aip;
        sig_rad     = sig0_pad * 
                      tan( point->GetDipAngle(i) ) /
                      cos( point->GetCrossAngle(i) ) ;
        sig_phi     = sig0_pad *
                      tan( point->GetCrossAngle(i) ) ;
        rad         = point->GetR(i);
        phi         = point->GetPhi(i);

        if (DEBUG) {
            cout << " ##### Point i= " << i
                 << " nel=" << electron 
                 << " rad=" << rad 
                 << " phi=" << phi 
                 << endl;
            cout << " #####            " 
                 << " sig_rad=" << sig_rad 
                 << " sig_phi=" << sig_phi 
                 << endl;
        }

        Cluster *cl = new Cluster(electron, sig_rad, sig_phi, 
                                  rad, phi, drift_time);

        while ( cl->IsDrifting(field) ) {
                cl->DriftDiffuse(field);
        }                                  // transport the cluster

        rdout->Avalanche(cl, field);       // multiply electrons

        rdout->PadResponse(cl);            // response on pad 

        rdout->ShaperResponse(cl, field);  // response on shaper 

        if (CLUSTER) rdout->BackProject(field, i, point);  
                                           // project cluster back to hit point

        rdout->Digitize(cl, ftpc);         // digitize signal

        if (DEBUG) {
            cout << "Gain*electron    = " << rdout->GetAElectron() << endl;
            cout << "sigma_pad [um]   = " << rdout->GetSigmaPad() << endl;
            cout << "sigma_tim [nsec] = " << rdout->GetSigmaTim() << endl;
        }

    } // end of loop over hit points

    if (PIXEL) {
        cout << "Writing out ADC array in file :" << file_adc << endl;
        rdout->OutputADC(file_adc);
    }

    if (CLUSTER) {
        cout << "Writing out cluster array in file :" << file_cluster << endl;
        point->OutputCluster(file_cluster);
    }
    
    return 0;
}



