///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#include "constant.h"

// for the random number generator
#define DIM 97

struct raset1 {
    float u[DIM];
    float c;
};

struct raset1 uc;



//////////////////////////////////////////

int Locate(const int npt, const fNumber* x, const fNumber xx)
{
//
// Locate the index in the array for a given value
// array x should be a monotonical array
//
    int jlow = 0;
    int jup  = npt-1;
    int jmid;
    int rising = (x[npt-1] > x[0]) ? (1) : (0) ;

    if (    rising  && (xx > x[jup] || xx < x[jlow])  ||
          (!rising) && (xx < x[jup] || xx > x[jlow]) ) {
         cout << "Locate(): xx is out of range!" << endl;
         exit(1);
    }

    while (jup-jlow-1 ) {
        jmid = (jlow + jup)/2;

        if (    rising  && (xx > x[jmid])  ||
              (!rising) && (xx < x[jmid]) ) {
            jlow = jmid;
        }
        else {
            jup  = jmid;
        }
    }

    return jlow;
}


//////////////////////////////////////////

fNumber Interpolate(const int npt, const fNumber* x, const fNumber* y, 
                    const int ich, const fNumber xx)
{
// 
// this subroutine will Interpolate the value (xx,yy) from
// the arrays {x(1:npt), y(1:npt)} at a given channel ich.
// 
    fNumber x1[5];
    fNumber y1[5];
    register int i;

    if (ich < 2) {
        for ( i=0; i<2; i++) {
            x1[i] = *(x+i) ;
            y1[i] = *(y+i) ;
        }
        return  InterpValue(2, x1, y1, xx);
    }
    else if (ich > npt-3) {
        for ( i=0; i<2; i++) {
            x1[i] = *(x+npt-1-i) ;
            y1[i] = *(y+npt-1-i) ;
        }
        return  InterpValue(2, x1, y1, xx);
    }
    else {
        for ( i=0; i<5; i++) {
            x1[i] = *(x+ich-2+i) ;
            y1[i] = *(y+ich-2+i) ;
        }
        return  InterpValue(5, x1, y1, xx);
    }
}

//////////////////////////////////////////

fNumber InterpValue(const int npt, const fNumber* x, const fNumber* y, 
                    const fNumber xx)
{
//
// use Newton's method to Interpolate value at x
//
    fNumber term;
    fNumber sum = 0;

    for(register int i=0; i < npt; i++) {
        term = y[i];
        for(register int j=0; j < npt; j++) {
            if (j != i) term *= (xx-x[j])/(x[i]-x[j]);
        }
        sum += term;
    }
    return sum;
}

//////////////////////////////////////////

fNumber InteGauss(const fNumber x_1, const fNumber x_2,
                  const fNumber x_0, const fNumber sig)
{

     fNumber x,x1,x2 ;

     x1 = (x_1-x_0) /sig;
     x2 = (x_2-x_0) /sig;
     if (x1 > x2) {
         x  = x2;  x2 = x1;  x1 = x;
     }

     fNumber del_x = (x2-x1)/((fNumber) (NSTEP-1) );

     // integrate the gauss function
     fNumber sum = 0;
     x = x1 + 0.5*del_x ;
     for( int i=0; i<(NSTEP-1); ++i ) {
         sum += exp(-0.5*x*x);
           x += del_x;
     }

     return del_x/sqrt(PI_2)*sum;
}

//////////////////////////////////////////

void rmarin(int ij, int kl)
{

/*   Initializing routine for RANMAR, must be called before */
/*   generating any psuedorandom numbers with RANMAR. The */
/*   input values should be in the ranges: */
/*       0 <= ij <= 31328 */
/*       0 <= kl <= 30081 */

/*   This shows correspondence between the simplified seeds */
/*   ij,kl and the original Marsaglia-Zaman seeds i,j,k,l */
/*   To get standard values in Marsaglia-Zaman paper */
/*   (i=12, j=34, k=56, l=78) put ij=1802, kl=9373. */

int ii, jj;
int i, j, k, l, m;
fNumber s, t;

i = (ij/177) % 177 + 2;
j = (ij) % 177 + 2;
k = (kl/169) % 178 + 1;
l = (kl) % 169;

// printf(" Ranmar initialized: %d %d %d %d %d %d \n",ij,kl,i,j,k,l);

cout << " Ranmar initialized:" << ij << " " 
                              << kl << " "
                              << i << " "
                              << j << " "
                              << k << " "
                              << l << endl;

for(ii=0; ii<DIM; ii++) {

    s = 0.0; 
    t = 0.5;

    for(jj=0; jj<24; jj++) {
        
        m =  ( (i*j) % 179 )*k % 179;
        i = j;
        j = k;
        k = m;
        l = (53*l+1) % 169;
        
        if ( (l*m)%64 >= 32 ) s += t;
        t *= 0.5;

    }

    uc.u[ii] = s;
    // printf(" ii = %d s= %f \n", ii,s);
}

uc.c = 362436./16777216.;

// printf(" c= %f \n", uc.c);

}

//////////////////////////////////////////

fNumber ranmar()
{
/* Universal random number generator proposed by Marsaglia */
/* and Zaman in report FSU-SCRI-87-50 */

/* From "A Review of Pseudorandom Number Generators" by */
/* F. James, CERN report SOFTWR 88-20. */

/* Rewritten as a function by Bill Long, 26-may-1989. */
/* Also modified to move cd and cm from initialization */
/* routine RMARIN to here as parameters. */

fNumber uni;
fNumber cd;
fNumber cm;
int i = DIM, j = 33;

cd = (fNumber) 7654321./(fNumber)16777216.;
cm = (fNumber)16777213./(fNumber)16777216.;

/*
printf(" cd = %20.17f; cm = %20.17f \n", cd, cm);
*/

uni = uc.u[i-1] - uc.u[j-1];
if (uni < (fNumber)0.0) uni += (fNumber)1.0;

uc.u[i-1] = uni;

--i;
if (i == 0) i = DIM;

--j;
if (j == 0 ) j = DIM;

uc.c -= cd;
if (uc.c < (fNumber)0.0) uc.c += cm;

uni -= uc.c;
if (uni < (fNumber)0.0) uni += (fNumber) 1.0;

return uni;
}
