///////////////////////////////////////////////////////////////////////////
//  Author: W.G.Gong
//  Email: gong@mppmu.mpg.de
//  Date:  Oct 25, 1996
///////////////////////////////////////////////////////////////////////////

#include "constant.h"
#include "inparam.h"
#include "point.h"
#include "chamber.h"
#include "field.h"
#include "cluster.h"
#include "readout.h"

Readout::Readout(const Inparam& inp)
{
    SetRandomNumberGen( (int) inp.GetInParam(Random_number_gen) );

    SetNumberPlane( (int) inp.GetInParam(Readout_number_plane) );
    SetPadPitch( inp.GetInParam(Readout_pad_pitch) );
    SetPadLength( inp.GetInParam(Readout_pad_length) );
    SetPadResponse( inp.GetInParam(Readout_sigma_prf) );   
    SetShaperTime( inp.GetInParam(Readout_shaper_time) );
    SetTimeSlice( inp.GetInParam(Readout_slice) );

    SetNPad(inp);

    // set parameters for Polya distribution and initialize it
    gnch   = 50;
    glow   = 0.1;
    ghigh  = 2.5; 
    gdelta = (ghigh-glow) / (fNumber) gnch;
    pcum   = new fNumber[gnch];
    polya(gnch, glow, ghigh, gdelta);

    ADC = new fNumber[NPAD*NTIM];
    for(register int i=0; i<NPAD*NTIM; ++i) ADC[i] = 0.;

    if (ran_num_gen == 1) {
        // initialize the random number generator
        rmarin(1802, 9373);
    }

}

Readout::Readout(const Inparam& inp, const int nplane, 
                 const fNumber pitch, const fNumber length, 
                 const fNumber prf, const fNumber shaper,
                 const fNumber slice) 
{
    SetNumberPlane(nplane);
    SetPadPitch(pitch);
    SetPadLength(length);
    SetPadResponse(prf);   
    SetShaperTime(shaper);
    SetTimeSlice(slice);

    SetNPad(inp);

    // set parameters for Polya distribution and initialize it
    gnch   = 50;
    glow   = 0.1;
    ghigh  = 2.5; 
    gdelta = (ghigh-glow) / (fNumber) gnch;
    pcum   = new fNumber[gnch];
    polya(gnch, glow, ghigh, gdelta);

    ADC = new fNumber[NPAD*NTIM];
    for(register int i=0; i<NPAD*NTIM; ++i) ADC[i] = 0.;
}

Readout::~Readout()
{
    delete [] ADC;
}

void  Readout::Avalanche(const Cluster *cl, const Field *field)
{
    // sample the gas gain according to Polya distribution

    fNumber gain = field->GetGasGain();
    int group = 10;                      // group 10 electrons together
    int nel   = (int) ( cl->GetElectron() / group );
    electrons = 0;
    for (int i=0; i<nel; ++i) {
        electrons += sample_polya(gain);
    }
    electrons *= group;

    //electrons = cl->GetElectron() * field->GetGasGain();

    // consider electron deflection due to Lorentz force
    // when electron approaches the strip
    // not implemented yet.
}

void  Readout::PadResponse(const Cluster *cl)
{
    fNumber prf = GetPadResponse();
    fNumber sig_phi = cl->GetSigPhi();
      
    sigma_pad =  sqrt(sig_phi*sig_phi + prf*prf ); 
}


void Readout::ShaperResponse(const Cluster *cl, const Field *field)
{
    fNumber srf = GetShaperResponse();
    fNumber sig_rad = cl->GetSigRad();

    // convert the radial width to nsec
    fNumber sig_time = sig_rad /  field->GetVelAtReadout();

    sigma_tim = sqrt( sig_time*sig_time + srf*srf) ;

}

void Readout::BackProject(const Field* field, const int i, Point* point)
{

    fNumber rad   = point->GetR(i);
    fNumber phi   = point->GetPhi(i);
    int     index = field->GetGridIndex(rad);

    // dr, dphi in [cm], projected back to hit position
    fNumber dr    = (sigma_tim*1.e-3) * field->GetVelocityZ(index);
                    //  [nsec]->[usec]     [cm/usec]
    fNumber dphi  = (sigma_pad*1.e-4) * (rad/field->GetRAtReadout()); 
                    //  [um]->[cm]         

    fNumber sin2phi  = sin(phi)*sin(phi);
    fNumber cos2phi  = 1. - sin2phi;
    if (cos2phi < 0.) cos2phi = 0.;

    fNumber dx = sqrt( dr*dr*cos2phi + dphi*dphi*sin2phi );
    fNumber dy = sqrt( dr*dr*sin2phi + dphi*dphi*cos2phi );
    
    point->SetDX(i, dx);
    point->SetDY(i, dy);

}

void Readout::Digitize(const Cluster *cl,  const Chamber *ftpc)
{
    fNumber FAC  = 5.0;        
    fNumber Rmax = ftpc->GetRMax();

    // get the readout position in radial direction
    fNumber time_slice = GetTimeSlice();
    fNumber time       = cl->GetDriftTime()*1000.;       // into nsec
    int     itim       = WhichSlice(time);

    // get the readout position in azimuthal direction
    fNumber delta_phi  = GetDeltaPhi(0, ftpc);            // in radian
    fNumber phi        = cl->GetPhi(0);
    int     ipad       = WhichPad(phi,ftpc);

 // big if() loop
 if ( itim > 2 && itim < (NTIM-3) &&
      ipad > 2 && ipad < (NPAD-3) ) {

      cout << "this cluster is at itim=" << itim 
           << ", ipad=" << ipad << endl;

    // and calculate the pad distribution

    fNumber sig_pad   = GetSigmaPad() / 10000.;  // into cm 
    fNumber width_phi = FAC *sig_pad / Rmax;

    int pad_min = WhichPad(phi-width_phi,ftpc); 
    int pad_max = WhichPad(phi+width_phi,ftpc); 
    int npad    = (pad_max - pad_min + 1);

    fNumber* pad = new fNumber[npad];  // signal dist. in pads
    for (int i=0; i<npad; ++i ) {
        fNumber phi_low = PhiOfPad(i+pad_min,0,ftpc) - 0.5*delta_phi; 
                                                 // low edge of pad
        fNumber phi_up  = PhiOfPad(i+pad_min,0,ftpc) + 0.5*delta_phi; 
                                                 // up  edge of pad
        pad[i] = InteGauss(Rmax*phi_low, Rmax*phi_up, 
                           Rmax*phi, sig_pad );
                                                 // integrate over this pad
    } // end for loop


    // and calculate the time distribution 
                
    fNumber sig_tim   = GetSigmaTim();
    fNumber width_tim = FAC*sig_tim;
    int tim_min = WhichSlice(time - width_tim);
    int tim_max = WhichSlice(time + width_tim);
    int ntim    = (tim_max - tim_min + 1);

    fNumber* sca = new fNumber[ntim];
    for (int j=0; j<ntim; ++j) {
        fNumber tim_low = TimeOfSlice(j+tim_min) 
                          - 0.5*time_slice; 
                                                 // low edge of time
        fNumber tim_up  = TimeOfSlice(j+tim_min) 
                          + 0.5*time_slice; 
                                                 // up edge of time

        sca[j] = InteGauss(tim_low, tim_up, time, sig_tim);
                                                 // integrate over this slice
    }  // end for loop

    // cout << "avalanched el=" << electrons << endl;

    // Now fill the ADC[pad,tim] array
    for (int i=0; i<npad; ++i) 
        for (int j=0; j<ntim; ++j) {
            int k = (i+pad_min)*NTIM + (j+tim_min) ;
            ADC[k] += (electrons * pad[i] * sca[j]) ; 
    }

    // recycle sca[] and pad[]
    delete [] sca;
    delete [] pad;

 } // end big if() loop

}
  

void Readout::OutputADC(const char* file_adc) const
{

    ofstream fout(file_adc);

    /*
    for (int pad=0; pad<NPAD; ++pad ) 
        for (int tim=0; tim<NTIM; ++tim ) {
            fNumber adc = ADC[pad][tim];
            if (adc) fout << pad <<", "<< tim <<", " << adc << endl;
    }*/
    fNumber adc ;
    for (int i=0; i<NPAD*NTIM; ++i) {
        int pad = i/NTIM;
        int tim = i - pad*NTIM;
        adc = ADC[i];

        if (adc > THRESHOLD ) {
            int iadc = (int) (adc/ADC_CONVERSION) ;
            // cout << "adc,iadc = " << adc <<"," << iadc << endl;
            if (iadc >= MAXADC) iadc=MAXADC;          // reset overflow
            if (iadc) fout << pad  <<"  " << tim <<"  " 
                           << iadc << endl;
        }
    }

    fout.close();

}

void Readout::SetNPad(const Inparam& inp)
{
    fNumber phi_min = inp.GetInParam(Chamber_phi_min)*PID;
    fNumber phi_max = inp.GetInParam(Chamber_phi_max)*PID;
    fNumber r_max   = inp.GetInParam(Chamber_r_max);
    fNumber pad = (phi_max-phi_min)*r_max/GetPadPitch() ;
    NPAD        = (int) pad;  
}

fNumber Readout::GetDeltaPhi(const int deg_or_rad, const Chamber *ftpc)
{
    return  ( deg_or_rad ? (GetPadPitch()/ftpc->GetRMax()*DPI) 
                         :  GetPadPitch()/ftpc->GetRMax()  ) ;
}

fNumber Readout::PhiOfPad(const int pad, const int deg_or_rad, 
                          const Chamber *ftpc)
{
    return  (0.5+pad) * GetDeltaPhi(deg_or_rad,ftpc);
}

int Readout::WhichPad(const fNumber phi, const Chamber *ftpc)
{
    // phi and phi_min in rad
    int ipad = (int) (ftpc->GetRMax()*(phi-ftpc->GetPhiMin(0))/GetPadPitch()) ;
    if (ipad < 0)  {
        ipad = 0;
        cout << "Readout: ipad < 0" << endl;
    }
    if (ipad > NPAD - 1) {
        ipad = NPAD - 1;
        cout << "Readout: ipad >NPAD - 1 " << endl;
    }
    return ipad;
}

int Readout::WhichSlice(const fNumber time)
{
    int itim = (int) (time/GetTimeSlice()) ;    // time in nsec
    if (itim < 0) {
        itim = 0;
        cout << "Readout: itim < 0 " << endl;
    }
    if (itim > NTIM - 1) {
        itim = NTIM - 1;
        cout << "Readout: itim >NTIM - 1 " << endl;
    }
    return itim;
}

fNumber Readout::TimeOfSlice(const int slice)
{
    return (slice+0.5)*GetTimeSlice();         // time in nsec
}

void Readout::Print() const 
{
    cout << " Number of pad rows = " 
         << GetNumberPlane() << endl;
    cout << " Number of pad per row = " 
         << GetNPad() << endl;
    cout << " Pad length = " 
         << GetPadLength() 
         << " pitch = " 
         << GetPadPitch() << " [cm]" << endl;
    cout << " Shaping time = " 
         << GetShaperTime() << " [ns]" << endl;
    cout << " Time slice = " 
         << GetTimeSlice() << " [ns]" << endl;
    cout << " Pad response sigma = " 
         << GetPadResponse() << " [um]" << endl;
                          
}


void Readout::polya(const int gnch, const fNumber glow, 
                    const fNumber ghigh, const fNumber gdelta)
{
//
// generate probability distribution from
// Polya function for gain fluctuation
// c.f.: Ronaldo Bellazzini and Mario Spezziga
//       La Rivista del Nuovo Cimento V17N12(1994)1.
//
//       m=3/2, gamma(m)=sqrt(PI)/2=0.8862269
//       polya(k) = m*pow((m*k),(m-1))*exp(-m*k)/gamma(m)
//
    fNumber m_polya = 1.5;
    fNumber c_polya = 1.6925687;

    fNumber x;
    fNumber p;
    pcum[0] = 0.0;
    for (int i=1; i<gnch; ++i) {
        x       = m_polya*(i*gdelta+glow);
        p       = c_polya*pow(x,(m_polya-1.0))*exp(-x);
        pcum[i] = pcum[i-1] + p ;
    }

    for (int i=0; i<gnch; ++i) {
        pcum[i] /= pcum[gnch-1];             // renormalize it
        //cout << "i=" << i << " pcum=" << pcum[i] << endl;
    }
}

int Readout::sample_polya(const fNumber gain)
{
    fNumber ran;
  
    if (ran_num_gen == 0) {
        ran = (fNumber) rand() / (fNumber) RAND_MAX;
    } else if (ran_num_gen == 1) {
        ran = ranmar();
    }

    int     ich = Locate(gnch, pcum, ran);
    //cout << "ich = " << ich << endl;
    return  (int) ( gain * ( glow + ich*gdelta ) );

}
